@@ -1,6 +1,6 @@
 # coding:utf-8
-import os
 from tools_hjh import other as tools
+import time
 
 
 class DBConn:
@@ -78,55 +78,64 @@ class DBConn:
             conn = self.dbpool.connection()
             
         cur = conn.cursor()
-                
-        # 执行SQL文件
-        if type(param) == str and os.path.exists(param):
-            file = open(param)
-            sqlStr = file.read()
-            file.close()
-            if self.dbtype == 'pgsql':
-                cur.execute(sqlStr)
-            elif self.dbtype == 'sqlite':
-                cur.executescript(sqlStr)
-            elif self.dbtype == 'mysql' or self.dbtype == 'oracle':
-                pass  # 暂不支持
-            conn.commit()
-            cur.close()
-            conn.close()
-            return self
         
-        # 执行非SELECT语句
-        if not sql.lower().strip().startswith("select"):
-            sql = sql.strip()
-            if type(param) == list:
-                cur.executemany(sql, param)
-            elif type(param) == tuple:
-                cur.execute(sql, param)
-            elif param is None:
-                cur.execute(sql)
-            conn.commit()
-            rownum = cur.rowcount
+        try:
+            '''
+            # 执行SQL文件
+            if type(param) == str and os.path.exists(param):
+                file = open(param)
+                sqlStr = file.read()
+                file.close()
+                if self.dbtype == 'pgsql':
+                    cur.execute(sqlStr)
+                elif self.dbtype == 'sqlite':
+                    cur.executescript(sqlStr)
+                elif self.dbtype == 'mysql' or self.dbtype == 'oracle':
+                    pass  # 暂不支持
+                conn.commit()
+                returnMess = self
+            '''
+            
+            # 执行非SELECT语句
+            if not sql.lower().strip().startswith("select"):
+                sql = sql.strip()
+                if type(param) == list:
+                    cur.executemany(sql, param)
+                elif type(param) == tuple:
+                    cur.execute(sql, param)
+                elif param is None:
+                    cur.execute(sql)
+                conn.commit()
+                rownum = cur.rowcount
+                return rownum
+        
+            # 执行SELECT语句
+            if sql.lower().strip().startswith("select"):
+                sql = sql.strip()
+                col = []
+                if param is None:
+                    cur.execute(sql)
+                elif type(param) == tuple:
+                    cur.execute(sql, param)
+                for c in cur.description:
+                    col.append(c[0])
+                rows = cur.fetchall()
+                return rs(tuple(col), rows)
+            
+        except Exception as e:
+            if self.dbtype == 'sqlite' and 'database is locked' in str(e):
+                cur.close()
+                conn.close()
+                time.sleep(1)
+                self.run(sql, param)
+            else:
+                raise e
+        finally:
             cur.close()
             conn.close()
-            return rownum
-    
-        # 执行SELECT语句
-        if sql.lower().strip().startswith("select"):
-            sql = sql.strip()
-            col = []
-            if param is None:
-                cur.execute(sql)
-            elif type(param) == tuple:
-                cur.execute(sql, param)
-            for c in cur.description:
-                col.append(c[0])
-            rows = cur.fetchall()
-            cur.close()
-            conn.close()
-            return {'cols':tuple(col), 'rows':rows}
         
     def insert(self, table, rows):
-        """ 往指定table中插入数据，rows是一个多个元组的列表，每个元组表示一组参数，或者是一个元组 """
+        """ 往指定table中插入数据，rows是一个多个元组的列表，每个元组表示一组参数；或者是一个元组 """
         paramNum = '?'
         if type(rows) == list and len(rows) > 0:
             for _ in range(len(rows[0]) - 1):
@@ -138,6 +147,8 @@ class DBConn:
                 paramNum = paramNum + ', ?'
             sql = 'insert into ' + table + ' values(' + paramNum + ')' 
             num = self.run(sql, rows)
+        else:
+            num = 0
         return num
                 
     def close(self):
@@ -176,7 +187,7 @@ class oracletools:
             from dba_tab_cols where owner = ? and table_name = ? and column_name not like '%$%' order by column_id
         '''
         tab = ''
-        cols_ = db.run(sql, (user, table))[1]
+        cols_ = db.run(sql, (user, table))["rows"]
         lenNum = 0
         for col_ in cols_:
             if lenNum < len(col_[0]):
@@ -213,7 +224,7 @@ class oracletools:
                 end column_type
             from dba_tab_cols where owner = ? and table_name = ? and column_name not like '%$%' order by column_id
         '''
-        for r in db.run(sql, (user, table))[1]:
+        for r in db.run(sql, (user, table))["rows"]:
             rssqls.append('alter table ' + user + '.' + table + ' add ' + r[0] + ' ' + r[1].strip())
         rssqls.append('alter table ' + user + '.' + table + ' drop column test_col')
         # 建主键
@@ -227,7 +238,7 @@ class oracletools:
             and t.constraint_type = 'P'
             group by t.constraint_name
         '''
-        for r in db.run(sql, (user, table))[1]:
+        for r in db.run(sql, (user, table))["rows"]:
             rssqls.append('alter table ' + user + '.' + table + ' add constraint ' + r[0] + ' primary key(' + r[1] + ')')
         # 建非空约束
         sql = '''
@@ -240,7 +251,7 @@ class oracletools:
             and t.constraint_type = 'C'
             and t.search_condition is not null
         '''
-        for r in db.run(sql, (user, table))[1]:
+        for r in db.run(sql, (user, table))["rows"]:
             if 'IS NOT NULL' in r[0]:
                 col = r[0].split(' ')[0]
                 rssqls.append('alter table ' + user + '.' + table + ' modify ' + col + ' not null')
@@ -255,7 +266,7 @@ class oracletools:
             and t.constraint_type = 'U'
             group by t.constraint_name
         '''
-        for r in db.run(sql, (user, table))[1]:
+        for r in db.run(sql, (user, table))["rows"]:
             rssqls.append('alter table ' + user + '.' + table + ' add constraint ' + r[0] + ' unique(' + r[1] + ')')
         # 建默认值
         sql = '''
@@ -266,7 +277,7 @@ class oracletools:
             and column_name not like '%$%'
             and data_default is not null
         '''
-        for r in db.run(sql, (user, table))[1]:
+        for r in db.run(sql, (user, table))["rows"]:
             rssqls.append('alter table ' + user + '.' + table + ' modify ' + r[0] + ' default ' + r[1].strip())
         # 建普通索引
         sql = '''
@@ -280,7 +291,7 @@ class oracletools:
             and t.index_type = 'NORMAL'
             group by t.index_name
         '''
-        for r in db.run(sql, (user, table))[1]:
+        for r in db.run(sql, (user, table))["rows"]:
             rssqls.append('create index ' + user + '.' + r[0] + ' on ' + user + '.' + table + '(' + r[1] + ')')
         # 建函数索引
         sql = '''
@@ -298,7 +309,7 @@ class oracletools:
         col, rows = db.run(sql, (user, table))
         mdb = MemoryDB()
         mdb.set('t_idx', col, rows)
-        rs = mdb.db.run('select index_name, group_concat(column_expression) from t_idx group by index_name')[1]
+        rs = mdb.db.run('select index_name, group_concat(column_expression) from t_idx group by index_name')["rows"]
         mdb.close()
         for r in rs:
             rssqls.append('create index ' + user + '.' + r[0] + ' on ' + user + '.' + table + '(' + r[1] + ')')
@@ -310,7 +321,7 @@ class oracletools:
             and table_name = ? 
             and comments is not null
         '''
-        for r in db.run(sql, (user, table))[1]:
+        for r in db.run(sql, (user, table))["rows"]:
             rssqls.append("comment on column " + user + "." + table + "." + r[0] + " is '" + r[1] + "'")
         # 建外键
         pass
@@ -326,7 +337,7 @@ class oracletools:
                 dbms_metadata.get_ddl('TABLE', ?, ?)
             ) from dual
         '''
-        return db.run(sql, (table, user))[1][0]
+        return db.run(sql, (table, user))["rows"][0]
     
     @staticmethod
     def compare_table(srcdb, srcuser, dstdb, dstuser):
@@ -336,7 +347,7 @@ class oracletools:
         out = ''
         tablist = []
         sql = 'select table_name from dba_tables where owner = ?'
-        srctabs = srcdb.run(sql, (srcuser,))[1]
+        srctabs = srcdb.run(sql, (srcuser,))["rows"]
         for tab in srctabs:
             srcdesc = oracletools.desc(srcdb, srcuser, tab[0])
             dstdesc = oracletools.desc(dstdb, dstuser, tab[0])
@@ -353,7 +364,7 @@ class oracletools:
         0：仅输出增量同步表结构的sql
         1：增量同步表结构
         2：重建表结构，不包含外键
-        3：重建表，且同步数据 """
+        3：重建表，且同步数据，数据量大的问题暂没考虑 """
         report = ''
         if mode == 0:
             sqls = oracletools.get_ddl(srcdb, srcuser, table)
@@ -386,7 +397,6 @@ class oracletools:
                 sql = 'drop table ' + dstuser + '.' + table + ' cascade constraints purge'
                 # dstdb.run(sql)
                 report = report + 'ok:' + sql + '\n'
-                
             except:
                 report = report + 'err:' + sql + '\n'
             sqls = oracletools.get_ddl(srcdb, srcuser, table)
@@ -448,3 +458,11 @@ class MemoryDB:
         self.close()
       
 
+class rs:
+    def __init__(self, cols, rows):
+        self.cols = cols
+        self.rows = rows
+    def getCols(self):
+        return self.cols
+    def getRows(self):
+        return self.rows
